@@ -24,54 +24,79 @@ public class LGConfig
 	private int maxCost = -1;
 	private boolean allowSkippedWords = true;
 	private String dictionaryLocation = null;
-	private boolean storeConstituentString = true;
+	private String language = null;
+
+	// mis-named; if flag is set, then a load is done, 
+	// not a store.
+	private boolean storeConstituentString = true; 
+	private boolean loadSense = false;
 	
 	public int getMaxLinkages()
 	{
 		return maxLinkages;
 	}
-	public void setMaxLinkages(int maxLinkages)
+	public void setMaxLinkages(int m)
 	{
-		this.maxLinkages = maxLinkages;
+		maxLinkages = m;
 	}
 	public int getMaxParseSeconds()
 	{
 		return maxParseSeconds;
 	}
-	public void setMaxParseSeconds(int maxParseSeconds)
+	public void setMaxParseSeconds(int m)
 	{
-		this.maxParseSeconds = maxParseSeconds;
+		maxParseSeconds = m;
 	}
 	public int getMaxCost()
 	{
 		return maxCost;
 	}
-	public void setMaxCost(int maxCost)
+	public void setMaxCost(int m)
 	{
-		this.maxCost = maxCost;
+		maxCost = m;
 	}
 	public boolean isAllowSkippedWords()
 	{
 		return allowSkippedWords;
 	}
-	public void setAllowSkippedWords(boolean allowSkippedWords)
+	public void setAllowSkippedWords(boolean a)
 	{
-		this.allowSkippedWords = allowSkippedWords;
+		allowSkippedWords = a;
 	}
 	public String getDictionaryLocation()
 	{
 		return dictionaryLocation;
 	}
-	public void setDictionaryLocation(String dictionaryLocation)
+	public void setDictionaryLocation(String d)
 	{
-		this.dictionaryLocation = dictionaryLocation;
+		dictionaryLocation = d;
 	}
+	public String getLanguage()
+	{
+		return language;
+	}
+	public void setLanguage(String l)
+	{
+		language = l;
+	}
+
+	// mis-named; if flag is set, then a load is done, 
+	// not a store.
 	public boolean isStoreConstituentString()
 	{
 		return storeConstituentString;
 	}
-	public void setStoreConstituentString(boolean storeConstituentString)
+	public void setStoreConstituentString(boolean s)
 	{
-		this.storeConstituentString = storeConstituentString;
+		storeConstituentString = s;
+	}	
+
+	public boolean isLoadSense()
+	{
+		return loadSense;
+	}
+	public void setLoadSense(boolean s)
+	{
+		loadSense = s;
 	}	
 }
