@@ -16,9 +16,10 @@ import java.util.List;
 /**
  * 
  * <p>
- * Represents the result of parsing a piece of text. The result consists of some global
- * meta information about the whole parse and a list of <code>Linkage</code>s returned
- * by the parser. The original parsed text is available as the <code>text</code> attribute
+ * Represents the result of parsing a piece of text. The result
+ * consists of some global meta information about the whole parse
+ * and a list of <code>Linkage</code>s returned by the parser. The
+ * original parsed text is available as the <code>text</code> attribute
  * and a tokenized version as the <code>String[] words</code> attribute.  
  * </p>
  *
@@ -28,6 +29,7 @@ import java.util.List;
 public class ParseResult implements Iterable<Linkage>
 {
 	String parserVersion;
+	String dictVersion;
 	String text;
 	String [] words;
 	boolean [] entityFlags;
@@ -35,17 +37,17 @@ public class ParseResult implements Iterable<Linkage>
 	List<Linkage> linkages = new ArrayList<Linkage>();
 	int numSkippedWords;
 	
-	public String wordAt(int i)
-	{
-		return words[i];
-	}
-	
-	public boolean isPastTenseForm(int i)
+	/**
+	 * past-tense verbs have a subscript of .v-d, .w-d or .q-d
+	 * look at the subscript instead, to see if a verb is past-tense.
+	 * @deprecated
+	 */
+	@Deprecated public boolean isPastTenseForm(int i)
 	{
 		return pastTenseFlags[i];
 	}
 	
-	public boolean isEntity(int i)
+	@Deprecated public boolean isEntity(int i)
 	{
 		return entityFlags[i];
 	}
@@ -70,32 +72,61 @@ public class ParseResult implements Iterable<Linkage>
 		this.text = text;
 	}
 
-	public String[] getWords()
+	/**
+	 * Use Linkage.wordAt instead.
+	 *
+	 * This method will return misleading results for languages
+	 * with prefixes/suffixes in the dictionary (such as Russian).
+	 * @deprecated
+	 */
+	@Deprecated public String wordAt(int i)
+	{
+		return words[i];
+	}
+	
+	/**
+	 * Use Linkage.getWords instead.
+	 *
+	 * This method will return misleading results for languages
+	 * with prefixes/suffixes in the dictionary (such as Russian).
+	 * @deprecated
+	 */
+	@Deprecated public String[] getWords()
 	{
 		return words;
 	}
 
-	public void setWords(String[] words)
+	@Deprecated public void setWords(String[] words)
 	{
 		this.words = words;
 	}
 
-	public boolean[] getEntityFlags()
+	@Deprecated public boolean[] getEntityFlags()
 	{
 		return entityFlags;
 	}
 
-	public void setEntityFlags(boolean[] entityFlags)
+	@Deprecated public void setEntityFlags(boolean[] entityFlags)
 	{
 		this.entityFlags = entityFlags;
 	}
 
-	public boolean[] getPastTenseFlags()
+	/**
+	 * past-tense verbs have a subscript of .v-d, .w-d or .q-d
+	 * look at the subscript instead, to see if a verb is past-tense.
+	 * @deprecated
+	 */
+	@Deprecated public boolean[] getPastTenseFlags()
 	{
 		return pastTenseFlags;
 	}
 
-	public void setPastTenseFlags(boolean[] pastTenseFlags)
+	/**
+	 * past-tense verbs have a subscript of .v-d, .w-d or .q-d
+	 * look at the subscript instead, to see if a verb is past-tense.
+	 * @deprecated
+	 */
+	@Deprecated public void setPastTenseFlags(boolean[] pastTenseFlags)
 	{
 		this.pastTenseFlags = pastTenseFlags;
 	}
@@ -119,4 +150,14 @@ public class ParseResult implements Iterable<Linkage>
 	{
 		this.parserVersion = parserVersion;
 	}	
+
+	public String getDictVersion()
+	{
+		return dictVersion;
+	}
+
+	public void setDictVersion(String dictVersion)
+	{
+		this.dictVersion = dictVersion;
+	}	
 }
