@@ -13,6 +13,7 @@
 
 #include "api.h"
 #include "disjunct-utils.h"
+#include "resources.h"
 
 #define CONTABSZ 8192
 typedef Connector * connector_table;
@@ -24,9 +25,7 @@ struct disjunct_dup_table_s
 	Disjunct ** dup_table;
 };
 
-/* the indiction in a word field that this connector cannot
- * be used -- is obsolete.
- */
+/* Indicator that this connector cannot be used -- that its "obsolete".  */
 #define BAD_WORD (MAX_SENTENCE+1)
 
 typedef struct c_list_s C_list;
@@ -1204,8 +1203,25 @@ static power_table * power_table_new(Sentence sent)
 
 	for (w=0; w<sent->length; w++)
 	{
+		/* The below uses variable-sized hash tables. This seems to
+		 * provide performance that is equal or better than the best
+		 * fixed-size prformance.
+		 * The best fixed-size performance seems to come at about 
+		 * a 1K table size, for both English and Russian. (Both have
+		 * about 100 fixed link-types, and many thousands of auto-genned
+		 * link types (IDxxx idioms for both, LLxxx suffix links for
+		 * Russian).  Plusses and minuses:
+		 * + small fixed tables are faster to initialize.
+		 * - small fixed tables have more collisions
+		 * - variable-size tables require counting connectors.
+		 *   (and the more complex code to go with)
+		 * CPU cache-size effects ...
+		 * Strong depenence on the hashing algo!
+		 */
 		len = left_connector_count(sent->word[w].d);
 		size = next_power_of_two_up(len);
+#define TOPSZ 32768
+		if (TOPSZ < size) size = TOPSZ;
 		pt->l_table_size[w] = size;
 		t = pt->l_table[w] = (C_list **) xalloc(size * sizeof(C_list *));
 		for (i=0; i<size; i++) t[i] = NULL;
@@ -1222,6 +1238,7 @@ static power_table * power_table_new(Sentence sent)
 
 		len = right_connector_count(sent->word[w].d);
 		size = next_power_of_two_up(len);
+		if (TOPSZ < size) size = TOPSZ;
 		pt->r_table_size[w] = size;
 		t = pt->r_table[w] = (C_list **) xalloc(size * sizeof(C_list *));
 		for (i=0; i<size; i++) t[i] = NULL;
@@ -1495,7 +1512,7 @@ int power_prune(Sentence sent, int mode, Parse_Options opts)
 	{
 		/* left-to-right pass */
 		for (w = 0; w < sent->length; w++) {
-			if (parse_options_resources_exhausted(opts)) break;
+			if ((2 == w%7) && parse_options_resources_exhausted(opts)) break;
 			for (d = sent->word[w].d; d != NULL; d = d->next) {
 				if (d->left == NULL) continue;
 				if (left_connector_list_update(pc, d->left, w, w, TRUE) < 0) {
@@ -1530,7 +1547,7 @@ int power_prune(Sentence sent, int mode, Parse_Options opts)
 		/* right-to-left pass */
 
 		for (w = sent->length-1; w >= 0; w--) {
-			if (parse_options_resources_exhausted(opts)) break;
+			if ((2 == w%7) && parse_options_resources_exhausted(opts)) break;
 			for (d = sent->word[w].d; d != NULL; d = d->next) {
 				if (d->right == NULL) continue;
 				if (right_connector_list_update(pc, sent, d->right, w, w, TRUE) >= sent->length) {
