@@ -184,13 +184,25 @@ strndup (const char *str, size_t size)
 /* ============================================================= */
 /* UTF8 utilities */
 
-int wctomb_check(char *s, wchar_t wc, mbstate_t *ps)
+int wctomb_check(char *s, wchar_t wc)
 {
-	int nr = wcrtomb(s, wc, ps);
+	int nr;
+#ifdef _MSC_VER
+	nr = WideCharToMultiByte(CP_UTF8, 0, &wc, 1, NULL, 0, NULL, NULL);
+	nr = WideCharToMultiByte(CP_UTF8, 0, &wc, 1, s, nr, NULL, NULL);
+	if (nr < 0) {
+		wprintf(L"Fatal Error: wctomb_check failed: %d %S\n", nr, &wc);
+		exit(1);
+	}
+#else
+	mbstate_t mbss;
+	memset(&mbss, 0, sizeof(mbss));
+	nr = wcrtomb(s, wc, &mbss);
 	if (nr < 0) {
 		prt_error("Fatal Error: unknown character set %s\n", nl_langinfo(CODESET));
 		exit(1);
 	}
+#endif
 	return nr;
 }
 
@@ -202,12 +214,15 @@ void downcase_utf8_str(char *to, const char * from, size_t usize)
 	wchar_t c;
 	int i, nbl, nbh;
 	char low[MB_LEN_MAX];
-	mbstate_t mbss;
 
+#ifdef _MSC_VER
+	nbh = MultiByteToWideChar(CP_UTF8, 0, from, 1, NULL, 0);
+	nbh = sizeof(wchar_t) * MultiByteToWideChar(CP_UTF8, 0, from, 1, &c, nbh);
+#else
 	nbh = mbtowc (&c, from, MB_CUR_MAX);
+#endif
 	c = towlower(c);
-	memset(&mbss, 0, sizeof(mbss));
-	nbl = wctomb_check(low, c, &mbss);
+	nbl = wctomb_check(low, c);
 
 	/* Check for error on an in-place copy */
 	if ((nbh < nbl) && (to == from))
@@ -235,12 +250,15 @@ void upcase_utf8_str(char *to, const char * from, size_t usize)
 	wchar_t c;
 	int i, nbl, nbh;
 	char low[MB_LEN_MAX];
-	mbstate_t mbss;
 
+#ifdef _MSC_VER
+	nbh = MultiByteToWideChar(CP_UTF8, 0, from, 1, NULL, 0);
+	nbh = sizeof(wchar_t) * MultiByteToWideChar(CP_UTF8, 0, from, 1, &c, nbh);
+#else
 	nbh = mbtowc (&c, from, MB_CUR_MAX);
+#endif
 	c = towupper(c);
-	memset(&mbss, 0, sizeof(mbss));
-	nbl = wctomb_check(low, c, &mbss);
+	nbl = wctomb_check(low, c);
 
 	/* Check for error on an in-place copy */
 	if ((nbh < nbl) && (to == from))
@@ -625,7 +643,10 @@ char * dictionary_get_data_dir(void)
 #ifdef _DEBUG
 				prt_error("Info: GetModuleFileName=%s", (prog_path ? prog_path : "NULL"));
 #endif
-				data_dir16 = path_get_dirname(prog_path);
+				if (sizeof(TCHAR) == 1)
+				    data_dir16 = path_get_dirname(prog_path16);
+				else
+				    data_dir16 = path_get_dirname(prog_path);
 				if (data_dir16 != NULL)
 				{
 					printf("   found dir for current prog %s\n", data_dir16);
@@ -791,21 +812,24 @@ FILE *dictopen(const char *filename, const char *how)
 /* ======================================================== */
 
 /**
- * Read in the whole stinkin file.
- * We read it as wide chars, one char at a time, so that any LC_CTYPE
- * setting are obeyed.  Later, during dictionary grokking, these will
- * be converted to the internal UTF8 format.  This routine returns
+ * Read in the whole stinkin file. This routine returns
  * malloced memory, which should be freed as soon as possible.
  */
-wchar_t *get_file_contents(const char * dict_name)
+char *get_file_contents(const char * dict_name)
 {
 	int fd;
 	size_t tot_size;
 	int left;
 	struct stat buf;
-	wchar_t * contents, *p;
+	char * contents, *p;
 
+#if defined(_MSC_VER) || defined(__MINGW32__)
+	/* binary, otherwise fstat file length is confused by crlf
+	 * counted as one byte. */
+	FILE *fp = dictopen(dict_name, "rb");
+#else
 	FILE *fp = dictopen(dict_name, "r");
+#endif
 	if (fp == NULL)
 		return NULL;
 
@@ -814,18 +838,14 @@ wchar_t *get_file_contents(const char * dict_name)
 	fstat(fd, &buf);
 	tot_size = buf.st_size;
 
-	/* This is surely more than we need. But that's OK, better
-	 * too much than too little.  That is, the file-size (measured
-	 * in bytes) over-estimate for the number of wide chars that
-	 * we expect to get out of it. 7 more for good luck. (wide chars)*/
-	contents = (wchar_t *) malloc(sizeof(wchar_t) * (tot_size+7));
+	contents = (char *) malloc(sizeof(char) * (tot_size+7));
 
 	/* Now, read the whole file. */
 	p = contents;
 	left = tot_size + 7;
 	while (1)
 	{
-		wchar_t *rv = fgetws(p, left, fp);
+		char *rv = fgets(p, left, fp);
 		if (NULL == rv || feof(fp))
 			break;
 		while (*p != 0x0) { p++; left--; }
