@@ -73,6 +73,36 @@ typedef enum
 	NO_LABEL=' '
 } Label;
 
+#if defined(_MSC_VER) || defined(__MINGW32__)
+/* Windows console (cmd.exe) input to utf8 */
+static char* oem_to_utf8(char *instring)
+{
+	char * out;
+	wchar_t *winput, *wp;
+	size_t len;
+	const char *p;
+	int cv;
+	unsigned int consolecp;
+
+	consolecp = GetConsoleOutputCP(); 
+
+	/* Convert input string to wide chars. */
+	len = strlen(instring) + 1;
+	cv = MultiByteToWideChar(consolecp, 0, instring, len, NULL, 0);
+	winput = (wchar_t*) malloc(cv * sizeof(wchar_t));
+	cv = MultiByteToWideChar(consolecp, 0, instring, len, winput, cv);
+
+	/* Convert wide chars to utf8. */
+	cv = WideCharToMultiByte(CP_UTF8, 0, winput, len, NULL, 0, NULL, NULL);
+	out = (char*) malloc(cv);
+	cv = WideCharToMultiByte(CP_UTF8, 0, winput, len, out, cv, NULL, NULL);
+
+	free(winput);
+
+	return out;
+}
+#endif
+
 static char *
 fget_input_string(FILE *in, FILE *out, Parse_Options opts)
 {
@@ -125,10 +155,53 @@ fget_input_string(FILE *in, FILE *out, Parse_Options opts)
 	}
 	input_pending = FALSE;
 
-	/* For UTF-8 input, I think its still technically correct to
-	 * use fgets() and not fgetws() at this point. */
+#if defined(_MSC_VER) || defined(__MINGW32__)
+	/* Windows console input comes using the console codepage;
+	 * convert it to utf8 */
+	if (stdin == in)
+	{
+		static char * pline = NULL;
+		if (fgets(input_string, MAX_INPUT, in))
+		{
+			char *cr, *lf;
+			if (pline) free(pline);
+			pline = oem_to_utf8(input_string);
+
+			cr = strchr(pline, '\r');
+			if (cr) *cr = '\0';
+			lf = strchr(pline, '\n');
+			if (lf) *lf = '\0';
+
+			return pline;
+		}
+	}
+	else
+	{
+		/* It appears that MS Win always provides wide chars, even if
+		 * one asked for "just a string".  So lets explicitly ask for
+		 * wide chars here, and convert to multi-byte UTF-8 on the fly.
+		 */
+		wchar_t winput_string[MAX_INPUT];
+		if (fgetws(winput_string, MAX_INPUT, in)) 
+		{
+			size_t nc = wcstombs(input_string, winput_string, MAX_INPUT);
+			if (nc && (((size_t) -1) != nc)) 
+			{
+				char *cr, *lf;
+				cr = strchr(input_string, '\r');
+				if (cr) *cr = '\0';
+				lf = strchr(input_string, '\n');
+				if (lf) *lf = '\0';
+
+				return input_string;
+			}
+		}
+	}
+#else
+	/* Linux et al return UTF-8 multi-byte strings. */
 	if (fgets(input_string, MAX_INPUT, in)) return input_string;
-	else return NULL;
+#endif
+	return NULL;
 #endif
 }
 
@@ -174,7 +247,7 @@ static int fget_input_char(FILE *in, FILE *out, Parse_Options opts)
 static void process_linkage(Linkage linkage, Parse_Options opts)
 {
 	char * string;
-	int mode;
+	ConstituentDisplayStyle mode;
 #ifdef USE_FAT_LINKAGES
 	int j, first_sublinkage;
 	int nlink;
@@ -561,6 +634,7 @@ int main(int argc, char * argv[])
 	int             num_linkages, i;
 	Label           label = NO_LABEL;
 	const char      *codeset;
+	const char      *locale = NULL;
 
 #if LATER
 	/* Try to catch the SIGWINCH ... except this is not working. */
@@ -579,10 +653,11 @@ int main(int argc, char * argv[])
 		i++;
 	}
 
+#if !defined(_MSC_VER) && !defined(__MINGW32__)
 	/* Get the locale from the environment... 
-	 * perhaps we should someday get it from the dictionary ??
+	 * Perhaps we should someday get it from the dictionary ??
 	 */
-	setlocale(LC_ALL, "");
+	locale = setlocale(LC_ALL, "");
 
 	/* Check to make sure the current locale is UTF8; if its not,
 	 * then force-set this to the english utf8 locale 
@@ -593,8 +668,14 @@ int main(int argc, char * argv[])
 		fprintf(stderr, 
 		    "%s: Warning: locale %s was not UTF-8; force-setting to en_US.UTF-8\n",
 		     argv[0], codeset);
-		setlocale(LC_CTYPE, "en_US.UTF-8");
+		locale = setlocale(LC_CTYPE, "en_US.UTF-8");
 	}
+#else
+ #pragma message("WARNING: Windows console (cmd.exe) does not support unicode input!\nWill attempt to convert from the native encoding!");
+	fprintf(stderr, 
+	    "%s: Warning: Windows console (cmd.exe) does not support unicode\n"
+	    "input!  Will attempt to convert from the native encoding!", argv[0]);
+#endif
 
 	for (; i<argc; i++)
 	{
@@ -649,7 +730,7 @@ int main(int argc, char * argv[])
 
 	if (dict == NULL)
 	{
-		fprintf(stderr, "%s: Fatal error: Unable to open  dictionary.\n", argv[0]);
+		fprintf(stderr, "%s: Fatal error: Unable to open dictionary.\n", argv[0]);
 		exit(-1);
 	}
 
@@ -685,6 +766,9 @@ int main(int argc, char * argv[])
 	parse_options_set_screen_width(opts, 79);
 #endif
 
+#if !defined(_MSC_VER) && !defined(__MINGW32__)
+	prt_error("Info: Using locale %s.", locale);
+#endif
 	prt_error("Info: Dictionary version %s.",
 		linkgrammar_get_dict_version(dict));
 	prt_error("Info: Library version %s. Enter \"!help\" for help.",
@@ -824,13 +908,13 @@ int main(int argc, char * argv[])
 				}
 			}
 
-			if (parse_options_timer_expired(opts))
+			if (verbosity > 0)
 			{
-				if (verbosity > 0) fprintf(stdout, "Timer is expired!\n");
-			}
-			if (parse_options_memory_exhausted(opts))
-			{
-				if (verbosity > 0) fprintf(stdout, "Memory is exhausted!\n");
+				if (parse_options_timer_expired(opts))
+					fprintf(stdout, "Timer is expired!\n");
+
+				if (parse_options_memory_exhausted(opts))
+					fprintf(stdout, "Memory is exhausted!\n");
 			}
 
 			if ((num_linkages == 0) &&
@@ -843,8 +927,10 @@ int main(int argc, char * argv[])
 				parse_options_reset_resources(panic_parse_opts);
 				parse_options_set_verbosity(panic_parse_opts, verbosity);
 				num_linkages = sentence_parse(sent, panic_parse_opts);
-				if (parse_options_timer_expired(panic_parse_opts)) {
-					if (verbosity > 0) fprintf(stdout, "Timer is expired!\n");
+				if (verbosity > 0)
+				{
+					if (parse_options_timer_expired(panic_parse_opts))
+						fprintf(stdout, "Panic timer is expired!\n");
 				}
 			}
 
