@@ -37,7 +37,7 @@ static Atom* normal_order(Atom* dnf)
 			Atom* a = ora->get_outgoing_atom(i);
 			norm.push_back(normal_order(a));
 		}
-		return new Or(norm);
+		return new Or(norm, ora->_tv);
 	}
 
 	And* andy = dynamic_cast<And*>(dnf);
@@ -60,8 +60,11 @@ static Atom* normal_order(Atom* dnf)
 			assert(c, "normal_order: expecting a connector in the disjunct");
 			if ('+' == c->get_direction())
 				norm.push_back(normal_order(c));
+
+			// Optional connectors are not allowed to appear in disjuncts!
+			assert(not c->is_optional(), "Optional connector in disjunct");
 		}
-		return new And(norm);
+		return new And(norm, andy->_tv);
 	}
 
 	// no-op
@@ -117,14 +120,21 @@ Atom* disjoin(Atom* mixed_form)
 			new_oset.push_back(norm);
 		}
 
-		Or* new_or = new Or(new_oset);
+		Or* new_or = new Or(new_oset, junct->_tv);
 		return normal_order(new_or->super_flatten());
 	}
 
 	And* junct = dynamic_cast<And*>(mixed_form);
-	assert(junct, "disjoin: given a naked AND link!");
+	assert(junct, "disjoin: mixed form is not AND link!");
 
 	junct = junct->flatten();
+	Atom* ajunct = junct->clean();
+
+	// After cleaning, it might be just a single optional clause.
+	// e.g. after (A+ or [[()]]) & B+;
+	junct = dynamic_cast<And*>(ajunct);
+	if (not junct)
+		return ajunct;
 
 	// If we are here, the outgoing set is a conjunction of atoms.
 	// Search for the first disjunction in that set, and distribute
@@ -166,24 +176,25 @@ Atom* disjoin(Atom* mixed_form)
 		OutList distrib;
 
 		// Copy the front, without change.
+		// We've got to clone the thing, because otherwise propagating
+		// the costs upwards becomes an ugly mess.
 		size_t jsz = front.size();
 		for (int j=0; j<jsz; j++)
-			distrib.push_back(front[j]);
+			distrib.push_back(front[j]->clone());
 
 		// insert one atom.
-		distrib.push_back(orn->get_outgoing_atom(i));
+		distrib.push_back(orn->get_outgoing_atom(i)->clone());
 
-		// Copy the rest.
+		// Copy the rest.  Again with the clone(). Irritating.
 		jsz = rest.size();
 		for (int j=0; j<jsz; j++)
-			distrib.push_back(rest[j]);
+			distrib.push_back(rest[j]->clone());
 
 		And *andy = new And(distrib);
-		andy = andy->clean();
-		new_oset.push_back(andy);
+		new_oset.push_back(andy->clean());
 	}
 
-	Or* new_or = new Or(new_oset);
+	Or* new_or = new Or(new_oset, orn->_tv);
 	Atom* new_a = new_or->super_flatten();
 	new_a = disjoin(new_a);
 
